#ifndef __cell_h_
#define __cell_h_
#include <adevs.h>
#include <fstream>

class cell: public atomic
{
	public:
		static const port_t in, out;
		static double D, dx, dy, k;
		static int width, height;

		static void create_space();
		static void delete_space();

		static void set_u(int x, int y, double value)
		{
			u[x][y] = value;
		}
		cell():atomic(){}
		void init()
		{
			q.setParams(D);
			double x0 = u[x][y];
			double dx0 = compute_derivative();
			q.init(&x0,&dx0);
			hold(q.ta());
		}
		void delta_int()
		{
			q.integ(q.ta());
			hold(q.ta());
		}
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x)
		{
			double dx = compute_derivative();
			q.integ(e,&dx);
			hold(q.ta());
		}
		void delta_conf(const adevs_bag<PortValue>& x) 
		{
			delta_ext(q.ta(),x);
		}
		void output_func(adevs_bag<PortValue>& _y)
		{
			output(out,NULL,_y);
			u[x][y] = q.out();
		}
		void gc_output(adevs_bag<PortValue>& g){}
		void report(ofstream& stream)
		{
			stream << q.state() << " ";
		}
		int get_x() const { return x; }
		int get_y() const { return y; }
		void set_pos(int x, int y) { this->x = x; this->y = y; } 
		~cell(){}

	private:	
		static double** u;
		qoden<1> q;
		int x, y;

		double compute_derivative()
		{
			// find neighboring cells using periodic boundary conditions
			// and appropriate offsets (for parallel computation)
			int xl = x-1;
			if (xl < 0) xl = width-1;
			int xr = (x+1)%width;
			int yu = y-1;
			if (yu < 0) yu = height-1;
			int yd = (y+1)%height;
			int xc = x;
			int yc = y;
			// compute new time derivative using center differences in space
			return (k*(u[xr][yc]-2.0*q.state()+u[xl][yc])/(dx*dx) +
						k*(u[xc][yu]-2.0*q.state()+u[xc][yd])/(dy*dy));
		}
};

#endif
