#ifndef __proc_h_
#define __proc_h_
#include <adevs.h>
#include <stdlib.h>
#include "job.h"
/*
A processor requires a fixed period of time to service a job.
The processor can serve only one job at a time.  It the processor
is busy, it simply discards incoming jobs.
*/
class proc: public atomic 
{
	public:
		proc(stime_t proc_time):
		atomic(),processing_time(proc_time),val(NULL){}
		void init()
		{
			passivate();
			if (val != NULL) delete val;
			val = NULL;
		}
		void delta_int()
		{
			passivate();
			if (val != NULL) delete val;
			val = NULL;
		}
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x)
		{
			if (timeNext() == INFINITY) 
			{
				val = new job(*((job*)x.get(0).value));
				hold(processing_time);
			}
		}
		void delta_conf(const adevs_bag<PortValue>& x)
		{
			delta_int();
			delta_ext(0.0,x);
		}
		void output_func(adevs_bag<PortValue>& y)
		{
			output(out,new job(*val),y);
		}
		void gc_output(adevs_bag<PortValue>& g)
		{
			for (int i = 0; i < g.getSize(); i++)
				delete g[i].value;
		}
		~proc()
		{
			if (val != NULL) delete val;
		}

		static const port_t in;
		static const port_t out;

	private:	
		stime_t processing_time;
		job* val;
};

const port_t proc::in(0);
const port_t proc::out(1);

#endif
