#ifndef __transd_h_
#define __transd_h_
#include <adevs.h>
#include <vector>
#include <iostream>
#include <stdlib.h>
#include "job.h"
/*
The transducer computes various statistics about the
performance of the queuing system.  It receives new jobs
on its ariv port, finished jobs on its solved port,
and generates and output on its out port when the observation
interval has elapsed.
*/
class transd: public atomic 
{
	public:
		transd(stime_t observ_time):
		atomic(),observation_time(observ_time),total_ta(0.0){}
		void init()
		{
			total_ta = 0.0;
			_arrived.clear();
			_solved.clear();
			hold(observation_time);
		}
		void delta_int() { passivate(); }
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x)
		{
			port_iterator ariv_port(x,ariv); 
			for (; !ariv_port.end_of_input(); ariv_port++) 
			{
				job val (*((job*)(*ariv_port)));
				val.t = timeCurrent();
				_arrived.push_back(val);
			}
			port_iterator solved_port(x,solved); 
			for (; !solved_port.end_of_input(); solved_port++) 
			{
				job val(*((job*)(*solved_port)));
				std::vector<job>::iterator i = _arrived.begin();
				for (; i != _arrived.end(); i++)
				{
					if ((*i).value == val.value)
					{
						total_ta += timeCurrent() - (*i).t;
						break;
					}
				}
				val.t = timeCurrent();
				_solved.push_back(val);
			}
		}
		void delta_conf(const adevs_bag<PortValue>& x)
		{
			delta_int();
			delta_ext(0.0,x);
		}
		void output_func(adevs_bag<PortValue>& y)
		{
			stime_t throughput;
			stime_t avg_ta_time;
			if (_solved.size() != 0) 
			{
				avg_ta_time = total_ta / _solved.size();
				if (timeCurrent() > 0)
					throughput = _solved.size() / timeCurrent ();
				else throughput = 0.0;
			}
			else 
			{
				avg_ta_time = 0.0;
				throughput = 0.0;
			}
			std::cout << "End time: " << timeCurrent() << std::endl;
			std::cout << "jobs arrived : " << _arrived.size () << std::endl;
			std::cout << "jobs solved : " << _solved.size() << std::endl;
			std::cout << "AVERAGE TA = " << avg_ta_time << std::endl;
			std::cout << "THROUGHPUT = " << throughput << std::endl;
			output(out,new dbl(throughput),y);
		}
		void gc_output(adevs_bag<PortValue>& g)
		{
			for (int i = 0; i < g.getSize(); i++)
				delete g[i].value;
		}
		~transd()
		{
			_arrived.clear();
			_solved.clear();
		}

		static const port_t ariv;
		static const port_t solved;
		static const port_t out;

	private:	
		std::vector<job> _arrived;
		std::vector<job> _solved; 
		stime_t observation_time;
		stime_t total_ta;
};

const port_t transd::ariv(0);
const port_t transd::solved(1);
const port_t transd::out(2);

#endif
